/**
 * Qt6 Common library
 *
 * The Qt6 Common library is auto-generated and available to users under the same license as the Qt which it is used with or built with. See available licenses below. 
 * SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

#ifndef QOAI_COMMON_GLOBAL_H
#define QOAI_COMMON_GLOBAL_H

#define OPENAPI_GENERATOR_VERSION_STR       "7.20.0"
#define OPENAPI_GENERATOR_VERSION_MAJOR     7
#define OPENAPI_GENERATOR_VERSION_MINOR     20
#define OPENAPI_GENERATOR_VERSION_PATCH     0

// Macro checks if the current generator version greater or equal
// to the provided value.
#define OPENAPI_GENERATOR_VERSION_GE(major, minor, patch) \
    ((OPENAPI_GENERATOR_VERSION_MAJOR > (major)) \
        || (OPENAPI_GENERATOR_VERSION_MAJOR == (major) \
            && OPENAPI_GENERATOR_VERSION_MINOR > (minor)) \
        || (OPENAPI_GENERATOR_VERSION_MAJOR == (major) \
            && OPENAPI_GENERATOR_VERSION_MINOR == (minor) \
                && OPENAPI_GENERATOR_VERSION_PATCH >= (patch)))

// Macro checks if the current generator version less or equal
// to the provided value.
#define OPENAPI_GENERATOR_VERSION_LE(major, minor, patch) \
    ((OPENAPI_GENERATOR_VERSION_MAJOR < (major)) \
        || (OPENAPI_GENERATOR_VERSION_MAJOR == (major) \
            && OPENAPI_GENERATOR_VERSION_MINOR < (minor)) \
        || (OPENAPI_GENERATOR_VERSION_MAJOR == (major) \
            && OPENAPI_GENERATOR_VERSION_MINOR == (minor) \
                && OPENAPI_GENERATOR_VERSION_PATCH <= (patch)))

#endif // QOAI_COMMON_GLOBAL_H
