/**
 * Qt6 Common library
 *
 * The Qt6 Common library is auto-generated and available to users under the same license as the Qt which it is used with or built with. See available licenses below. 
 * SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

#ifndef OPENAPICOMMON_COMMON_EXPORTS_H
#define OPENAPICOMMON_COMMON_EXPORTS_H

#include <QtCore/qtconfigmacros.h>

#if defined(OPENAPICOMMON_COMMON_LIB_SHARED) || !defined(OPENAPICOMMON_COMMON_LIB_STATIC)
#  if defined(OPENAPICOMMON_BUILD_COMMON_LIB)
#    define OPENAPICOMMON_COMMON_EXPORT Q_DECL_EXPORT
#  else
#    define OPENAPICOMMON_COMMON_EXPORT Q_DECL_IMPORT
#  endif
#else
#  define OPENAPICOMMON_COMMON_EXPORT
#endif

#endif // OPENAPICOMMON_COMMON_EXPORTS_H
